/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.stream.impl.delegator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.ow2.util.file.FileUtils;
import org.ow2.util.file.FileUtilsException;
import org.ow2.util.stream.api.IStreamAccessor;

/**
 * This class copy a stream from a stream accessor to a local file.
 * Then all local file modifications will be reported to the stream
 * accessor. However modifications of stream accessor will not be reported
 * to local file.
 * @author Gael Lalire
 */
public class DelegatorStreamAccessor implements IStreamAccessor {

    private static final int BUFFER_SIZE = 8192;

    private File delegatorFile;

    private Object mutex = new Object();

    private IStreamAccessor parentFileAccessor;

    public DelegatorStreamAccessor(final IStreamAccessor parentFileAccessor) throws IOException {
        try {
            initDelegatorFileAccessor(parentFileAccessor.getInputStream());
        } catch (FileNotFoundException e) {
        }
        this.parentFileAccessor = parentFileAccessor;
    }

    protected DelegatorStreamAccessor(final IStreamAccessor parentFileAccessor, final InputStream inputStream)
            throws IOException {
        initDelegatorFileAccessor(inputStream);
        this.parentFileAccessor = parentFileAccessor;
    }

    private void initDelegatorFileAccessor(final InputStream inputStream)
            throws IOException {
        if (inputStream == null) {
            delegatorFile = null;
        } else {
            delegatorFile = File.createTempFile("DelegatorFileAccessor", ".data");
            byte[] buf = new byte[BUFFER_SIZE];
            OutputStream outputStream = new FileOutputStream(delegatorFile);
            int read = inputStream.read(buf);
            while (read != -1) {
                outputStream.write(buf, 0, read);
                read = inputStream.read(buf);
            }
            inputStream.close();
            outputStream.close();
        }
    }

    protected void writeToParent(final IStreamAccessor parentFileAccessor, final InputStream inputStream) throws IOException {
        OutputStream parentOutputStream = parentFileAccessor.getOutputStream();
        byte[] buf = new byte[BUFFER_SIZE];
        int read = inputStream.read(buf);
        while (read != -1) {
            parentOutputStream.write(buf, 0, read);
            read = inputStream.read(buf);
        }
        parentOutputStream.flush();
    }

    protected final void doWrite() throws IOException {
        synchronized (mutex) {
            if (delegatorFile == null) {
                throw new FileNotFoundException("unable to write, the file was delete");
            }
            writeToParent(parentFileAccessor, new FileInputStream(delegatorFile));
        }
    }

    protected final void doWrite(final File temporaryFile) throws IOException {
        synchronized (mutex) {
            writeToParent(parentFileAccessor, new FileInputStream(temporaryFile));
            if (delegatorFile == null) {
                delegatorFile = File.createTempFile("DelegatorFileAccessor", ".data");
                try {
                    FileUtils.copyFile(temporaryFile, delegatorFile);
                } catch (FileUtilsException e) {
                    throw new IOException("unable to copy file");
                }
            }
        }
    }

    public InputStream getInputStream() throws IOException {
        synchronized (mutex) {
            if (delegatorFile == null) {
                throw new FileNotFoundException();
            }
            return new DelegatorInputStream(this, delegatorFile);
        }
    }

    public OutputStream getOutputStream() throws IOException {
        synchronized (mutex) {
            if (delegatorFile == null) {
                delegatorFile = File.createTempFile("DelegatorFileAccessor", ".data");
            }
            return new DelegatorOutputStream(this, delegatorFile, false);
        }
    }

    public OutputStream getTemporaryOutputStream() throws IOException {
        return new DelegatorOutputStream(this, File.createTempFile("DelegatorFileAccessor", ".data"), true);
    }

    @Override
    protected void finalize() {
        synchronized (mutex) {
            if (delegatorFile != null) {
                delegatorFile.delete();
            }
        }
    }

    public void delete() {
        synchronized (mutex) {
            if (delegatorFile != null) {
                delegatorFile.delete();
                delegatorFile = null;
            }
        }
        parentFileAccessor.delete();
    }

    public boolean exists() {
        synchronized (mutex) {
            if (delegatorFile != null) {
                return true;
            }
            return false;
        }
    }

}
