/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.stream.impl.delegator.zip;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.ow2.util.stream.api.IStreamAccessor;
import org.ow2.util.stream.impl.delegator.DelegatorStreamAccessor;

/**
 * @author Gael Lalire
 */
public class DelegatorZipStreamAccessor extends DelegatorStreamAccessor implements IStreamAccessor {

    private static final int BUFFER_SIZE = 8192;

    private String fileName;

    private static InputStream getInputStream(final IStreamAccessor parentFileAccessor, final String fileName)
            throws IOException {
        InputStream parentInputStream;
        try {
            parentInputStream = parentFileAccessor.getInputStream();
        } catch (FileNotFoundException e) {
            return null;
        }
        ZipInputStream zipInputStream = new ZipInputStream(parentInputStream);
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        if (zipEntry == null) {
            return null;
        }
        while (!zipEntry.getName().equals(fileName)) {
            zipEntry = zipInputStream.getNextEntry();
            if (zipEntry == null) {
                return null; // a null input stream mean that file does not
                             // exists
            }
        }
        return zipInputStream;
    }

    public DelegatorZipStreamAccessor(final IStreamAccessor parentFileAccessor, final String fileName) throws ZipException,
            IOException {
        super(parentFileAccessor, getInputStream(parentFileAccessor, fileName)); // the file will be extracted in a tmp file
        this.fileName = fileName;
    }

    @Override
    protected void writeToParent(final IStreamAccessor parentFileAccessor, final InputStream localFileInputStream)
            throws IOException {
        ZipOutputStream zipOutputStream = new ZipOutputStream(parentFileAccessor.getTemporaryOutputStream());
        InputStream parentInputStream = null;
        try {
            parentInputStream = parentFileAccessor.getInputStream();
        } catch (FileNotFoundException e) {
        }
        byte[] buf = new byte[BUFFER_SIZE];
        boolean fileFound = false;

        if (parentInputStream != null) {
            ZipInputStream zipInputStream = new ZipInputStream(parentInputStream);
            ZipEntry zipEntry = zipInputStream.getNextEntry();

            while (zipEntry != null) {
                InputStream inputStream;
                boolean replaceZipFile = zipEntry.getName().equals(fileName);
                if (!replaceZipFile) {
                    // keep other file in zip
                    inputStream = zipInputStream;
                } else {
                    // replace file in zip
                    zipEntry = new ZipEntry(fileName);
                    inputStream = localFileInputStream;
                    fileFound = true;
                }
                zipOutputStream.putNextEntry(zipEntry);
                int read = inputStream.read(buf);
                while (read != -1) {
                    zipOutputStream.write(buf, 0, read);
                    read = inputStream.read(buf);
                }
                if (replaceZipFile) {
                    inputStream.close();
                } else {
                    zipInputStream.closeEntry();
                }
                zipOutputStream.closeEntry();
                zipEntry = zipInputStream.getNextEntry();
            }
            zipInputStream.close();
        }
        // add file in zip
        if (!fileFound) {
            ZipEntry newZipEntry = new ZipEntry(fileName);
            zipOutputStream.putNextEntry(newZipEntry);

            int read = localFileInputStream.read(buf);
            while (read != -1) {
                zipOutputStream.write(buf, 0, read);
                read = localFileInputStream.read(buf);
            }
            localFileInputStream.close();
            zipOutputStream.closeEntry();
        }

        // close input stream before
        zipOutputStream.close();
    }

}
